@@ -45,8 +45,8 @@ type statePermutation struct {
 func newStatePermutationFromBuild(build *common.Build) statePermutation {
 	return statePermutation{
 		runner:        build.Runner.ShortDescription(),
-		buildState:    build.CurrentState,
-		buildStage:    build.CurrentStage,
+		buildState:    build.CurrentState(),
+		buildStage:    build.CurrentStage(),
 		executorStage: build.CurrentExecutorStage(),
 	}
 }
@@ -193,7 +193,9 @@ func (b *buildsHelper) removeBuild(deleteBuild *common.Build) bool {
 	b.lock.Lock()
 	defer b.lock.Unlock()
 
-	b.jobDurationHistogram.WithLabelValues(deleteBuild.Runner.ShortDescription()).Observe(deleteBuild.Duration().Seconds())
+	b.jobDurationHistogram.
+		WithLabelValues(deleteBuild.Runner.ShortDescription()).
+		Observe(deleteBuild.Duration().Seconds())
 
 	for idx, build := range b.builds {
 		if build == deleteBuild {
@@ -291,10 +293,14 @@ func (b *buildsHelper) ListJobsHandler(w http.ResponseWriter, r *http.Request) {
 	for _, job := range b.builds {
 		url := CreateJobURL(job.RepoCleanURL(), job.ID)
 
-		fmt.Fprintf(
+		_, _ = fmt.Fprintf(
 			w,
 			"url=%s state=%s stage=%s executor_stage=%s duration=%s\n",
-			url, job.CurrentState, job.CurrentStage, job.CurrentExecutorStage(), job.Duration(),
+			url,
+			job.CurrentState(),
+			job.CurrentStage(),
+			job.CurrentExecutorStage(),
+			job.Duration(),
 		)
 	}
 }
