@@ -362,6 +362,7 @@ func TestJobFailure(t *testing.T) {
 	executor.On("Shell").Return(&ShellScriptInfo{Shell: "script-shell"})
 	executor.On("Run", matchBuildStage(BuildStagePrepare)).Return(nil).Once()
 	executor.On("Run", mock.Anything).Return(thrownErr).Times(2)
+	executor.On("Run", matchBuildStage(BuildStageCleanupFileVariables)).Return(nil).Once()
 	executor.On("Finish", thrownErr).Once()
 
 	RegisterExecutorProvider("build-run-job-failure", provider)
@@ -444,6 +445,7 @@ func TestRunFailureRunsAfterScriptAndArtifactsOnFailure(t *testing.T) {
 	executor.On("Run", matchBuildStage("step_script")).Return(errors.New("build fail")).Once()
 	executor.On("Run", matchBuildStage(BuildStageAfterScript)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageUploadOnFailureArtifacts)).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageCleanupFileVariables)).Return(nil).Once()
 	executor.On("Finish", errors.New("build fail")).Once()
 
 	RegisterExecutorProvider("build-run-run-failure", provider)
@@ -475,6 +477,7 @@ func TestGetSourcesRunFailure(t *testing.T) {
 	executor.On("Run", matchBuildStage(BuildStagePrepare)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageGetSources)).Return(errors.New("build fail")).Times(3)
 	executor.On("Run", matchBuildStage(BuildStageUploadOnFailureArtifacts)).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageCleanupFileVariables)).Return(nil).Once()
 	executor.On("Finish", errors.New("build fail")).Once()
 
 	build := registerExecutorWithSuccessfulBuild(t, provider, new(RunnerConfig))
@@ -498,6 +501,7 @@ func TestArtifactDownloadRunFailure(t *testing.T) {
 	executor.On("Run", matchBuildStage(BuildStageRestoreCache)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageDownloadArtifacts)).Return(errors.New("build fail")).Times(3)
 	executor.On("Run", matchBuildStage(BuildStageUploadOnFailureArtifacts)).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageCleanupFileVariables)).Return(nil).Once()
 	executor.On("Finish", errors.New("build fail")).Once()
 
 	build := registerExecutorWithSuccessfulBuild(t, provider, new(RunnerConfig))
@@ -515,7 +519,7 @@ func TestArtifactUploadRunFailure(t *testing.T) {
 	executor.On("Cleanup").Once()
 
 	// Successful build script
-	executor.On("Shell").Return(&ShellScriptInfo{Shell: "script-shell"}).Times(8)
+	executor.On("Shell").Return(&ShellScriptInfo{Shell: "script-shell"}).Times(9)
 	executor.On("Run", matchBuildStage(BuildStagePrepare)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageGetSources)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageRestoreCache)).Return(nil).Once()
@@ -524,6 +528,7 @@ func TestArtifactUploadRunFailure(t *testing.T) {
 	executor.On("Run", matchBuildStage(BuildStageAfterScript)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageArchiveCache)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageUploadOnSuccessArtifacts)).Return(errors.New("upload fail")).Once()
+	executor.On("Run", matchBuildStage(BuildStageCleanupFileVariables)).Return(nil).Once()
 	executor.On("Finish", errors.New("upload fail")).Once()
 
 	build := registerExecutorWithSuccessfulBuild(t, provider, new(RunnerConfig))
@@ -553,6 +558,7 @@ func TestRestoreCacheRunFailure(t *testing.T) {
 	executor.On("Run", matchBuildStage(BuildStageGetSources)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageRestoreCache)).Return(errors.New("build fail")).Times(3)
 	executor.On("Run", matchBuildStage(BuildStageUploadOnFailureArtifacts)).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageCleanupFileVariables)).Return(nil).Once()
 	executor.On("Finish", errors.New("build fail")).Once()
 
 	build := registerExecutorWithSuccessfulBuild(t, provider, new(RunnerConfig))
@@ -1541,6 +1547,9 @@ func setupSuccessfulMockExecutor(
 	executor.On("Run", matchBuildStage(BuildStageUploadOnSuccessArtifacts)).
 		Return(nil).
 		Once()
+	executor.On("Run", matchBuildStage(BuildStageCleanupFileVariables)).
+		Return(nil).
+		Once()
 
 	return provider, assertFn
 }
