@@ -98,7 +98,8 @@ func GetRemoteSuccessfulLFSBuild() (JobResponse, error) {
 
 func GetRemoteSuccessfulBuildWithAfterScript() (JobResponse, error) {
 	jobResponse, err := GetRemoteBuildResponse("echo Hello World")
-	jobResponse.Steps = append(jobResponse.Steps,
+	jobResponse.Steps = append(
+		jobResponse.Steps,
 		Step{
 			Name:   StepNameAfterScript,
 			Script: []string{"echo Hello World"},
@@ -114,7 +115,8 @@ func GetRemoteSuccessfulMultistepBuild() (JobResponse, error) {
 		return JobResponse{}, err
 	}
 
-	jobResponse.Steps = append(jobResponse.Steps,
+	jobResponse.Steps = append(
+		jobResponse.Steps,
 		Step{
 			Name:   "release",
 			Script: []string{"echo Release"},
@@ -183,6 +185,47 @@ func GetRemoteLongRunningBuild() (JobResponse, error) {
 	return GetRemoteBuildResponse("sleep 3600")
 }
 
+func GetRemoteLongRunningBuildCMD() (JobResponse, error) {
+	// Can't use TIMEOUT since it requires input redirection,
+	// https://knowledge.broadcom.com/external/article/29524/the-timeout-command-in-batch-script-job.html
+	return GetLocalBuildResponse("ping 127.0.0.1 -n 3600 > nul")
+}
+
+func GetRemoteLongRunningBuildWithAfterScript() (JobResponse, error) {
+	jobResponse, err := GetRemoteLongRunningBuild()
+	if err != nil {
+		return JobResponse{}, err
+	}
+
+	addAfterScript(&jobResponse)
+
+	return jobResponse, nil
+}
+
+func GetRemoteLongRunningBuildWithAfterScriptCMD() (JobResponse, error) {
+	jobResponse, err := GetRemoteLongRunningBuildCMD()
+	if err != nil {
+		return JobResponse{}, err
+	}
+
+	addAfterScript(&jobResponse)
+
+	return jobResponse, nil
+}
+
+func addAfterScript(jobResponse *JobResponse) {
+	jobResponse.Steps = append(
+		jobResponse.Steps,
+		Step{
+			Name: StepNameAfterScript,
+			Script: []string{
+				"echo Hello World from after_script",
+			},
+			When: StepWhenAlways,
+		},
+	)
+}
+
 func GetMultilineBashBuild() (JobResponse, error) {
 	return GetRemoteBuildResponse(`if true; then
 	echo 'Hello World'
@@ -225,9 +268,11 @@ func getRemoteCustomTLSBuild(chain string) (JobResponse, error) {
 	}
 
 	job.TLSCAChain = chain
-	job.Variables = append(job.Variables,
+	job.Variables = append(
+		job.Variables,
 		JobVariable{Key: "GIT_STRATEGY", Value: "clone"},
-		JobVariable{Key: "GIT_SUBMODULE_STRATEGY", Value: "normal"})
+		JobVariable{Key: "GIT_SUBMODULE_STRATEGY", Value: "normal"},
+	)
 
 	return job, nil
 }
@@ -253,6 +298,9 @@ func GetRemoteBuildResponse(commands ...string) (JobResponse, error) {
 func GetLocalBuildResponse(commands ...string) (JobResponse, error) {
 	localRepoURL, err := getLocalRepoURL()
 	if err != nil {
+		if os.IsNotExist(err) {
+			panic("Local repo not found, please run `make development_setup`")
+		}
 		return JobResponse{}, err
 	}
 
@@ -323,7 +371,7 @@ func getGitLabComTLSChain() (string, error) {
 	if err != nil {
 		return "", err
 	}
-	defer resp.Body.Close()
+	defer func() { _ = resp.Body.Close() }()
 
 	var buff bytes.Buffer
 	for _, certs := range resp.TLS.VerifiedChains {
