@@ -4,6 +4,7 @@ import (
 	"bytes"
 	"context"
 	"crypto/md5"
+	"crypto/sha1"
 	"errors"
 	"fmt"
 	"io"
@@ -68,19 +69,8 @@ func (s *safeBuffer) String() string {
 	return s.buf.String()
 }
 
-func TestDockerCommandSuccessRun(t *testing.T) {
-	helpers.SkipIntegrationTests(t, "docker", "info")
-	test.SkipIfGitLabCIOn(t, test.OSWindows)
-
-	build := getBuildForOS(t, common.GetRemoteSuccessfulBuild)
-
-	err := build.Run(&common.Config{}, &common.Trace{Writer: os.Stdout})
-	assert.NoError(t, err)
-}
-
 func TestDockerCommandMultistepBuild(t *testing.T) {
 	helpers.SkipIntegrationTests(t, "docker", "info")
-	test.SkipIfGitLabCIOn(t, test.OSWindows)
 
 	tests := map[string]struct {
 		buildGetter    func() (common.JobResponse, error)
@@ -142,6 +132,7 @@ func TestDockerCommandMultistepBuild(t *testing.T) {
 			if tt.errExpected {
 				var buildErr *common.BuildError
 				assert.True(t, errors.As(err, &buildErr), "expected %T, got %T", buildErr, err)
+				assert.Contains(t, err.Error(), "exit code 1")
 				return
 			}
 			assert.NoError(t, err)
@@ -150,6 +141,16 @@ func TestDockerCommandMultistepBuild(t *testing.T) {
 }
 
 func getBuildForOS(t *testing.T, getJobResp func() (common.JobResponse, error)) common.Build {
+	jobResp, err := getJobResp()
+	require.NoError(t, err)
+
+	return common.Build{
+		JobResponse: jobResp,
+		Runner:      getRunnerConfigForOS(t),
+	}
+}
+
+func getRunnerConfigForOS(t *testing.T) *common.RunnerConfig {
 	executor := "docker"
 	image := common.TestAlpineImage
 
@@ -158,23 +159,17 @@ func getBuildForOS(t *testing.T, getJobResp func() (common.JobResponse, error))
 		image = getWindowsImage(t)
 	}
 
-	jobResp, err := getJobResp()
-	require.NoError(t, err)
-
-	return common.Build{
-		JobResponse: jobResp,
-		Runner: &common.RunnerConfig{
-			RunnerSettings: common.RunnerSettings{
-				Executor: executor,
-				Docker: &common.DockerConfig{
-					Image:      image,
-					PullPolicy: common.PullPolicyIfNotPresent,
-				},
-			},
-			RunnerCredentials: common.RunnerCredentials{
-				Token: fmt.Sprintf("%x", md5.Sum([]byte(t.Name()))),
+	return &common.RunnerConfig{
+		RunnerSettings: common.RunnerSettings{
+			Executor: executor,
+			Docker: &common.DockerConfig{
+				Image:      image,
+				PullPolicy: common.PullPolicyIfNotPresent,
 			},
 		},
+		RunnerCredentials: common.RunnerCredentials{
+			Token: fmt.Sprintf("%x", md5.Sum([]byte(t.Name()))),
+		},
 	}
 }
 
@@ -197,7 +192,6 @@ func getWindowsImage(t *testing.T) string {
 
 func TestDockerCommandSuccessRunRawVariable(t *testing.T) {
 	helpers.SkipIntegrationTests(t, "docker", "info")
-	test.SkipIfGitLabCIOn(t, test.OSWindows)
 
 	build := getBuildForOS(t, func() (common.JobResponse, error) {
 		return common.GetRemoteBuildResponse("echo $TEST")
@@ -215,9 +209,31 @@ func TestDockerCommandSuccessRunRawVariable(t *testing.T) {
 	assert.Contains(t, out, value)
 }
 
+func TestDockerCommandSuccessRunFileVariableContent(t *testing.T) {
+	helpers.SkipIntegrationTests(t, "docker", "info")
+
+	build := getBuildForOS(t, func() (common.JobResponse, error) {
+		if runtime.GOOS == "windows" {
+			return common.GetRemoteBuildResponse(`Get-Filehash -Algorithm SHA1 -Path $TEST`)
+		}
+		return common.GetRemoteBuildResponse(`sha1sum $TEST | tr "[a-z]" "[A-Z]"`)
+	})
+
+	value := "this is the content"
+	build.Variables = append(build.Variables, common.JobVariable{
+		Key:   "TEST",
+		Value: value,
+		File:  true,
+		Raw:   true,
+	})
+
+	out, err := buildtest.RunBuildReturningOutput(t, &build)
+	assert.NoError(t, err)
+	assert.Contains(t, out, fmt.Sprintf("%X", sha1.Sum([]byte(value))))
+}
+
 func TestDockerCommandUsingCustomClonePath(t *testing.T) {
 	helpers.SkipIntegrationTests(t, "docker", "info")
-	test.SkipIfGitLabCIOn(t, test.OSWindows)
 
 	remoteBuild := func() (common.JobResponse, error) {
 		cmd := "ls -al $CI_BUILDS_DIR/go/src/gitlab.com/gitlab-org/repo"
@@ -285,18 +301,6 @@ func TestDockerCommandNoRootImage(t *testing.T) {
 	assert.NoError(t, err)
 }
 
-func TestDockerCommandBuildFail(t *testing.T) {
-	helpers.SkipIntegrationTests(t, "docker", "info")
-	test.SkipIfGitLabCIOn(t, test.OSWindows)
-
-	build := getBuildForOS(t, common.GetRemoteFailedBuild)
-
-	err := build.Run(&common.Config{}, &common.Trace{Writer: os.Stdout})
-	require.Error(t, err, "error")
-	assert.IsType(t, &common.BuildError{}, err)
-	assert.Contains(t, err.Error(), "exit code 1")
-}
-
 func TestDockerCommandWithAllowedImagesRun(t *testing.T) {
 	helpers.SkipIntegrationTests(t, "docker", "info")
 	test.SkipIfGitLabCIOn(t, test.OSWindows)
@@ -457,66 +461,12 @@ func TestDockerCommandMissingTag(t *testing.T) {
 	assert.Contains(t, err.Error(), "not found")
 }
 
-func TestDockerCommandBuildAbort(t *testing.T) {
-	helpers.SkipIntegrationTests(t, "docker", "info")
-	test.SkipIfGitLabCIOn(t, test.OSWindows)
-
-	longRunningBuild, err := common.GetRemoteLongRunningBuild()
-	assert.NoError(t, err)
-	build := &common.Build{
-		JobResponse: longRunningBuild,
-		Runner: &common.RunnerConfig{
-			RunnerSettings: common.RunnerSettings{
-				Executor: "docker",
-				Docker: &common.DockerConfig{
-					Image:      common.TestAlpineImage,
-					PullPolicy: common.PullPolicyIfNotPresent,
-				},
-			},
-		},
-		SystemInterrupt: make(chan os.Signal, 1),
-	}
-
-	abortTimer := time.AfterFunc(time.Second, func() {
-		t.Log("Interrupt")
-		build.SystemInterrupt <- os.Interrupt
-	})
-	defer abortTimer.Stop()
-
-	timeoutTimer := time.AfterFunc(time.Minute, func() {
-		t.Log("Timedout")
-		t.FailNow()
-	})
-	defer timeoutTimer.Stop()
-
-	err = build.Run(&common.Config{}, &common.Trace{Writer: os.Stdout})
-	assert.EqualError(t, err, "aborted: interrupt")
-}
-
 func TestDockerCommandBuildCancel(t *testing.T) {
 	if helpers.SkipIntegrationTests(t, "docker", "info") {
 		return
 	}
 
-	build := getBuildForOS(t, common.GetRemoteLongRunningBuild)
-
-	trace := &common.Trace{Writer: os.Stdout}
-
-	abortTimer := time.AfterFunc(time.Second, func() {
-		t.Log("Interrupt")
-		trace.Cancel()
-	})
-	defer abortTimer.Stop()
-
-	timeoutTimer := time.AfterFunc(time.Minute, func() {
-		t.Log("Timedout")
-		t.FailNow()
-	})
-	defer timeoutTimer.Stop()
-
-	err := build.Run(&common.Config{}, trace)
-	assert.IsType(t, &common.BuildError{}, err)
-	assert.Contains(t, err.Error(), "canceled")
+	buildtest.RunBuildWithCancel(t, getRunnerConfigForOS(t), nil)
 }
 
 func TestDockerCommandTwoServicesFromOneImage(t *testing.T) {
@@ -1266,7 +1216,9 @@ func TestDockerCommandWithHelperImageConfig(t *testing.T) {
 		t,
 		out,
 		"Using docker image sha256:3cf24b1b62b6a4c55c5de43db4f50c0ff8b455238c836945d4b5c645411bfc77 for "+
-			"gitlab/gitlab-runner-helper:x86_64-5a147c92 ...")
+			"gitlab/gitlab-runner-helper:x86_64-5a147c92 with digest "+
+			"gitlab/gitlab-runner-helper@sha256:836120f351e800cd483402a5910b0a347f9e32e97ac3e94e67e2c005a289cc4c ...",
+	)
 }
 
 func TestDockerCommandWithDoingPruneAndAfterScript(t *testing.T) {
