@@ -3,7 +3,6 @@ package parallels_test
 import (
 	"os"
 	"testing"
-	"time"
 
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
@@ -162,80 +161,21 @@ func TestParallelsMissingSSHCredentials(t *testing.T) {
 	assert.Contains(t, err.Error(), "missing SSH config")
 }
 
-func TestParallelsBuildAbort(t *testing.T) {
-	if helpers.SkipIntegrationTests(t, prlCtl, "--version") {
-		return
-	}
-
-	longRunningBuild, err := common.GetRemoteLongRunningBuild()
-	assert.NoError(t, err)
-	build := &common.Build{
-		JobResponse: longRunningBuild,
-		Runner: &common.RunnerConfig{
-			RunnerSettings: common.RunnerSettings{
-				Executor: "parallels",
-				Parallels: &common.ParallelsConfig{
-					BaseName:         prlImage,
-					DisableSnapshots: true,
-				},
-				SSH: prlSSHConfig,
-			},
-		},
-		SystemInterrupt: make(chan os.Signal, 1),
-	}
-
-	abortTimer := time.AfterFunc(time.Second, func() {
-		t.Log("Interrupt")
-		build.SystemInterrupt <- os.Interrupt
-	})
-	defer abortTimer.Stop()
-
-	timeoutTimer := time.AfterFunc(time.Minute, func() {
-		t.Log("Timedout")
-		t.FailNow()
-	})
-	defer timeoutTimer.Stop()
-
-	err = build.Run(&common.Config{}, &common.Trace{Writer: os.Stdout})
-	assert.EqualError(t, err, "aborted: interrupt")
-}
-
 func TestParallelsBuildCancel(t *testing.T) {
 	if helpers.SkipIntegrationTests(t, prlCtl, "--version") {
 		return
 	}
 
-	longRunningBuild, err := common.GetRemoteLongRunningBuild()
-	assert.NoError(t, err)
-	build := &common.Build{
-		JobResponse: longRunningBuild,
-		Runner: &common.RunnerConfig{
-			RunnerSettings: common.RunnerSettings{
-				Executor: "parallels",
-				Parallels: &common.ParallelsConfig{
-					BaseName:         prlImage,
-					DisableSnapshots: true,
-				},
-				SSH: prlSSHConfig,
+	config := &common.RunnerConfig{
+		RunnerSettings: common.RunnerSettings{
+			Executor: "parallels",
+			Parallels: &common.ParallelsConfig{
+				BaseName:         prlImage,
+				DisableSnapshots: true,
 			},
+			SSH: prlSSHConfig,
 		},
 	}
 
-	trace := &common.Trace{Writer: os.Stdout}
-
-	abortTimer := time.AfterFunc(time.Second, func() {
-		t.Log("Interrupt")
-		trace.Cancel()
-	})
-	defer abortTimer.Stop()
-
-	timeoutTimer := time.AfterFunc(time.Minute, func() {
-		t.Log("Timedout")
-		t.FailNow()
-	})
-	defer timeoutTimer.Stop()
-
-	err = build.Run(&common.Config{}, trace)
-	assert.IsType(t, &common.BuildError{}, err)
-	assert.EqualError(t, err, "canceled")
+	buildtest.RunBuildWithCancel(t, config, nil)
 }
