@@ -103,7 +103,13 @@ func (n *GitLabClient) getClient(credentials requestCredentials) (c *client, err
 	if n.clients == nil {
 		n.clients = make(map[string]*client)
 	}
-	key := fmt.Sprintf("%s_%s_%s_%s", credentials.GetURL(), credentials.GetToken(), credentials.GetTLSCAFile(), credentials.GetTLSCertFile())
+	key := fmt.Sprintf(
+		"%s_%s_%s_%s",
+		credentials.GetURL(),
+		credentials.GetToken(),
+		credentials.GetTLSCAFile(),
+		credentials.GetTLSCertFile(),
+	)
 	c = n.clients[key]
 	if c == nil {
 		c, err = newClient(credentials)
@@ -124,6 +130,12 @@ func (n *GitLabClient) getLastUpdate(credentials requestCredentials) (lu string)
 	return cli.getLastUpdate()
 }
 
+// getFeatures enables features that are properties of networking client
+func (n *GitLabClient) getFeatures(features *common.FeaturesInfo) {
+	features.TraceReset = true
+	features.TraceChecksum = true
+}
+
 func (n *GitLabClient) getRunnerVersion(config common.RunnerConfig) common.VersionInfo {
 	info := common.VersionInfo{
 		Name:         common.NAME,
@@ -135,8 +147,10 @@ func (n *GitLabClient) getRunnerVersion(config common.RunnerConfig) common.Versi
 		Shell:        config.Shell,
 	}
 
+	n.getFeatures(&info.Features)
+
 	if executorProvider := common.GetExecutorProvider(config.Executor); executorProvider != nil {
-		executorProvider.GetFeatures(&info.Features)
+		_ = executorProvider.GetFeatures(&info.Features)
 
 		if info.Shell == "" {
 			info.Shell = executorProvider.GetDefaultShell()
@@ -150,7 +164,13 @@ func (n *GitLabClient) getRunnerVersion(config common.RunnerConfig) common.Versi
 	return info
 }
 
-func (n *GitLabClient) doRaw(credentials requestCredentials, method, uri string, request io.Reader, requestType string, headers http.Header) (res *http.Response, err error) {
+func (n *GitLabClient) doRaw(
+	credentials requestCredentials,
+	method, uri string,
+	request io.Reader,
+	requestType string,
+	headers http.Header,
+) (res *http.Response, err error) {
 	c, err := n.getClient(credentials)
 	if err != nil {
 		return nil, err
@@ -159,7 +179,13 @@ func (n *GitLabClient) doRaw(credentials requestCredentials, method, uri string,
 	return c.do(uri, method, request, requestType, headers)
 }
 
-func (n *GitLabClient) doJSON(credentials requestCredentials, method, uri string, statusCode int, request interface{}, response interface{}) (int, string, *http.Response) {
+func (n *GitLabClient) doJSON(
+	credentials requestCredentials,
+	method, uri string,
+	statusCode int,
+	request interface{},
+	response interface{},
+) (int, string, *http.Response) {
 	c, err := n.getClient(credentials)
 	if err != nil {
 		return clientError, err.Error(), nil
@@ -168,7 +194,10 @@ func (n *GitLabClient) doJSON(credentials requestCredentials, method, uri string
 	return c.doJSON(uri, method, statusCode, request, response)
 }
 
-func (n *GitLabClient) getResponseTLSData(credentials requestCredentials, response *http.Response) (ResponseTLSData, error) {
+func (n *GitLabClient) getResponseTLSData(
+	credentials requestCredentials,
+	response *http.Response,
+) (ResponseTLSData, error) {
 	c, err := n.getClient(credentials)
 	if err != nil {
 		return ResponseTLSData{}, fmt.Errorf("couldn't get client: %w", err)
@@ -177,7 +206,10 @@ func (n *GitLabClient) getResponseTLSData(credentials requestCredentials, respon
 	return c.getResponseTLSData(response.TLS)
 }
 
-func (n *GitLabClient) RegisterRunner(runner common.RunnerCredentials, parameters common.RegisterRunnerParameters) *common.RegisterRunnerResponse {
+func (n *GitLabClient) RegisterRunner(
+	runner common.RunnerCredentials,
+	parameters common.RegisterRunnerParameters,
+) *common.RegisterRunnerResponse {
 	// TODO: pass executor
 	request := common.RegisterRunnerRequest{
 		RegisterRunnerParameters: parameters,
@@ -186,9 +218,16 @@ func (n *GitLabClient) RegisterRunner(runner common.RunnerCredentials, parameter
 	}
 
 	var response common.RegisterRunnerResponse
-	result, statusText, resp := n.doJSON(&runner, http.MethodPost, "runners", http.StatusCreated, &request, &response)
+	result, statusText, resp := n.doJSON(
+		&runner,
+		http.MethodPost,
+		"runners",
+		http.StatusCreated,
+		&request,
+		&response,
+	)
 	if resp != nil {
-		defer resp.Body.Close()
+		defer func() { _ = resp.Body.Close() }()
 	}
 
 	switch result {
@@ -214,7 +253,7 @@ func (n *GitLabClient) VerifyRunner(runner common.RunnerCredentials) bool {
 
 	result, statusText, resp := n.doJSON(&runner, http.MethodPost, "runners/verify", http.StatusOK, &request, nil)
 	if resp != nil {
-		defer resp.Body.Close()
+		defer func() { _ = resp.Body.Close() }()
 	}
 
 	switch result {
@@ -241,7 +280,7 @@ func (n *GitLabClient) UnregisterRunner(runner common.RunnerCredentials) bool {
 
 	result, statusText, resp := n.doJSON(&runner, http.MethodDelete, "runners", http.StatusNoContent, &request, nil)
 	if resp != nil {
-		defer resp.Body.Close()
+		defer func() { _ = resp.Body.Close() }()
 	}
 
 	const baseLogText = "Unregistering runner from GitLab"
@@ -278,7 +317,10 @@ func addTLSData(response *common.JobResponse, tlsData ResponseTLSData) {
 	}
 }
 
-func (n *GitLabClient) RequestJob(config common.RunnerConfig, sessionInfo *common.SessionInfo) (*common.JobResponse, bool) {
+func (n *GitLabClient) RequestJob(
+	config common.RunnerConfig,
+	sessionInfo *common.SessionInfo,
+) (*common.JobResponse, bool) {
 	request := common.JobRequest{
 		Info:       n.getRunnerVersion(config),
 		Token:      config.Token,
@@ -287,7 +329,14 @@ func (n *GitLabClient) RequestJob(config common.RunnerConfig, sessionInfo *commo
 	}
 
 	var response common.JobResponse
-	result, statusText, httpResponse := n.doJSON(&config.RunnerCredentials, http.MethodPost, "jobs/request", http.StatusCreated, &request, &response)
+	result, statusText, httpResponse := n.doJSON(
+		&config.RunnerCredentials,
+		http.MethodPost,
+		"jobs/request",
+		http.StatusCreated,
+		&request,
+		&response,
+	)
 
 	n.requestsStatusesMap.Append(config.RunnerCredentials.ShortDescription(), APIEndpointRequestJob, result)
 
@@ -321,15 +370,20 @@ func (n *GitLabClient) RequestJob(config common.RunnerConfig, sessionInfo *commo
 	}
 }
 
-func (n *GitLabClient) UpdateJob(config common.RunnerConfig, jobCredentials *common.JobCredentials, jobInfo common.UpdateJobInfo) common.UpdateState {
+func (n *GitLabClient) UpdateJob(
+	config common.RunnerConfig,
+	jobCredentials *common.JobCredentials,
+	jobInfo common.UpdateJobInfo,
+) common.UpdateJobResult {
 	request := common.UpdateJobRequest{
 		Info:          n.getRunnerVersion(config),
 		Token:         jobCredentials.Token,
 		State:         jobInfo.State,
 		FailureReason: jobInfo.FailureReason,
+		Checksum:      jobInfo.Checksum,
 	}
 
-	result, statusText, response := n.doJSON(
+	statusCode, statusText, response := n.doJSON(
 		&config.RunnerCredentials,
 		http.MethodPut,
 		fmt.Sprintf("jobs/%d", jobInfo.ID),
@@ -337,44 +391,73 @@ func (n *GitLabClient) UpdateJob(config common.RunnerConfig, jobCredentials *com
 		&request,
 		nil,
 	)
-	n.requestsStatusesMap.Append(config.RunnerCredentials.ShortDescription(), APIEndpointUpdateJob, result)
+	n.requestsStatusesMap.Append(config.RunnerCredentials.ShortDescription(), APIEndpointUpdateJob, statusCode)
 
-	remoteJobStateResponse := NewRemoteJobStateResponse(response)
-	log := config.Log().WithFields(logrus.Fields{
-		"code":       result,
-		"job":        jobInfo.ID,
-		"job-status": remoteJobStateResponse.RemoteState,
+	log := config.Log().WithField("job", jobInfo.ID)
+
+	return n.createUpdateJobResult(log, statusCode, statusText, response)
+}
+
+func (n *GitLabClient) createUpdateJobResult(
+	log *logrus.Entry,
+	statusCode int,
+	statusText string,
+	response *http.Response,
+) common.UpdateJobResult {
+	remoteJobStateResponse := NewRemoteJobStateResponse(response, log)
+
+	result := common.UpdateJobResult{
+		NewUpdateInterval: remoteJobStateResponse.RemoteUpdateInterval,
+	}
+
+	log = log.WithFields(logrus.Fields{
+		"code":            statusCode,
+		"job-status":      remoteJobStateResponse.RemoteState,
+		"update-interval": remoteJobStateResponse.RemoteUpdateInterval,
 	})
 
 	switch {
 	case remoteJobStateResponse.IsAborted():
 		log.Warningln("Submitting job to coordinator...", "aborted")
-		return common.UpdateAbort
-	case result == http.StatusOK:
+		result.State = common.UpdateAbort
+	case statusCode == http.StatusOK:
 		log.Debugln("Submitting job to coordinator...", "ok")
-		return common.UpdateSucceeded
-	case result == http.StatusNotFound:
+		result.State = common.UpdateSucceeded
+	case statusCode == http.StatusAccepted:
+		log.Debugln("Submitting job to coordinator...", "accepted, but not yet completed")
+		result.State = common.UpdateAcceptedButNotCompleted
+	case statusCode == http.StatusPreconditionFailed:
+		log.Debugln("Submitting job to coordinator...", "trace validation failed")
+		result.State = common.UpdateTraceValidationFailed
+	case statusCode == http.StatusNotFound:
 		log.Warningln("Submitting job to coordinator...", "aborted")
-		return common.UpdateAbort
-	case result == http.StatusForbidden:
+		result.State = common.UpdateAbort
+	case statusCode == http.StatusForbidden:
 		log.WithField("status", statusText).Errorln("Submitting job to coordinator...", "forbidden")
-		return common.UpdateAbort
-	case result == clientError:
+		result.State = common.UpdateAbort
+	case statusCode == clientError:
 		log.WithField("status", statusText).Errorln("Submitting job to coordinator...", "error")
-		return common.UpdateAbort
+		result.State = common.UpdateAbort
 	default:
 		log.WithField("status", statusText).Warningln("Submitting job to coordinator...", "failed")
-		return common.UpdateFailed
+		result.State = common.UpdateFailed
 	}
+
+	return result
 }
 
-func (n *GitLabClient) PatchTrace(config common.RunnerConfig, jobCredentials *common.JobCredentials, content []byte, startOffset int) common.PatchTraceResult {
+func (n *GitLabClient) PatchTrace(
+	config common.RunnerConfig,
+	jobCredentials *common.JobCredentials,
+	content []byte,
+	startOffset int,
+) common.PatchTraceResult {
 	id := jobCredentials.ID
 
 	baseLog := config.Log().WithField("job", id)
 	if len(content) == 0 {
 		baseLog.Debugln("Appending trace to coordinator...", "skipped due to empty patch")
-		return common.NewPatchTraceResult(startOffset, common.UpdateSucceeded, 0)
+		return common.NewPatchTraceResult(startOffset, common.PatchSucceeded, 0)
 	}
 
 	endOffset := startOffset + len(content)
@@ -390,13 +473,19 @@ func (n *GitLabClient) PatchTrace(config common.RunnerConfig, jobCredentials *co
 	response, err := n.doRaw(&config.RunnerCredentials, "PATCH", uri, request, "text/plain", headers)
 	if err != nil {
 		config.Log().Errorln("Appending trace to coordinator...", "error", err.Error())
-		return common.NewPatchTraceResult(startOffset, common.UpdateFailed, 0)
+		return common.NewPatchTraceResult(startOffset, common.PatchFailed, 0)
 	}
 
-	n.requestsStatusesMap.Append(config.RunnerCredentials.ShortDescription(), APIEndpointPatchTrace, response.StatusCode)
+	n.requestsStatusesMap.Append(
+		config.RunnerCredentials.ShortDescription(),
+		APIEndpointPatchTrace,
+		response.StatusCode,
+	)
 
-	defer response.Body.Close()
-	defer io.Copy(ioutil.Discard, response.Body)
+	defer func() {
+		_, _ = io.Copy(ioutil.Discard, response.Body)
+		_ = response.Body.Close()
+	}()
 
 	tracePatchResponse := NewTracePatchResponse(response, baseLog)
 	log := baseLog.WithFields(logrus.Fields{
@@ -405,50 +494,60 @@ func (n *GitLabClient) PatchTrace(config common.RunnerConfig, jobCredentials *co
 		"job-status":      tracePatchResponse.RemoteState,
 		"code":            response.StatusCode,
 		"status":          response.Status,
-		"update-interval": tracePatchResponse.RemoteTraceUpdateInterval,
+		"update-interval": tracePatchResponse.RemoteUpdateInterval,
 	})
 
+	return n.createPatchTraceResult(startOffset, tracePatchResponse, response, endOffset, log)
+}
+
+func (n *GitLabClient) createPatchTraceResult(
+	startOffset int,
+	tracePatchResponse *TracePatchResponse,
+	response *http.Response,
+	endOffset int,
+	log *logrus.Entry,
+) common.PatchTraceResult {
 	result := common.PatchTraceResult{
 		SentOffset:        startOffset,
-		NewUpdateInterval: tracePatchResponse.RemoteTraceUpdateInterval,
+		NewUpdateInterval: tracePatchResponse.RemoteUpdateInterval,
 	}
 
 	switch {
 	case tracePatchResponse.IsAborted():
 		log.Warningln("Appending trace to coordinator...", "aborted")
-		result.State = common.UpdateAbort
+		result.State = common.PatchAbort
 
 		return result
 
 	case response.StatusCode == http.StatusAccepted:
 		log.Debugln("Appending trace to coordinator...", "ok")
 		result.SentOffset = endOffset
-		result.State = common.UpdateSucceeded
+		result.State = common.PatchSucceeded
 
 		return result
 
 	case response.StatusCode == http.StatusNotFound:
 		log.Warningln("Appending trace to coordinator...", "not-found")
-		result.State = common.UpdateNotFound
+		result.State = common.PatchNotFound
 
 		return result
 
 	case response.StatusCode == http.StatusRequestedRangeNotSatisfiable:
 		log.Warningln("Appending trace to coordinator...", "range mismatch")
 		result.SentOffset = tracePatchResponse.NewOffset()
-		result.State = common.UpdateRangeMismatch
+		result.State = common.PatchRangeMismatch
 
 		return result
 
 	case response.StatusCode == clientError:
 		log.Errorln("Appending trace to coordinator...", "error")
-		result.State = common.UpdateAbort
+		result.State = common.PatchAbort
 
 		return result
 
 	default:
 		log.Warningln("Appending trace to coordinator...", "failed")
-		result.State = common.UpdateFailed
+		result.State = common.PatchFailed
 
 		return result
 	}
@@ -485,18 +584,24 @@ func uploadRawArtifactsQuery(options common.ArtifactsOptions) url.Values {
 	return q
 }
 
-func (n *GitLabClient) UploadRawArtifacts(config common.JobCredentials, reader io.Reader, options common.ArtifactsOptions) common.UploadState {
+func (n *GitLabClient) UploadRawArtifacts(
+	config common.JobCredentials,
+	reader io.Reader,
+	options common.ArtifactsOptions,
+) common.UploadState {
 	pr, pw := io.Pipe()
-	defer pr.Close()
+	defer func() { _ = pr.Close() }()
 
 	mpw := multipart.NewWriter(pw)
 
 	go func() {
-		defer pw.Close()
-		defer mpw.Close()
+		defer func() {
+			_ = mpw.Close()
+			_ = pw.Close()
+		}()
 		err := n.createArtifactsForm(mpw, reader, options.BaseName)
 		if err != nil {
-			pw.CloseWithError(err)
+			_ = pw.CloseWithError(err)
 		}
 	}()
 
@@ -504,7 +609,14 @@ func (n *GitLabClient) UploadRawArtifacts(config common.JobCredentials, reader i
 
 	headers := make(http.Header)
 	headers.Set("JOB-TOKEN", config.Token)
-	res, err := n.doRaw(&config, http.MethodPost, fmt.Sprintf("jobs/%d/artifacts?%s", config.ID, query.Encode()), pr, mpw.FormDataContentType(), headers)
+	res, err := n.doRaw(
+		&config,
+		http.MethodPost,
+		fmt.Sprintf("jobs/%d/artifacts?%s", config.ID, query.Encode()),
+		pr,
+		mpw.FormDataContentType(),
+		headers,
+	)
 
 	log := logrus.WithFields(logrus.Fields{
 		"id":    config.ID,
@@ -524,29 +636,43 @@ func (n *GitLabClient) UploadRawArtifacts(config common.JobCredentials, reader i
 		log.WithError(err).Errorln(messagePrefix, "error")
 		return common.UploadFailed
 	}
-	defer res.Body.Close()
-	defer io.Copy(ioutil.Discard, res.Body)
+	defer func() {
+		_, _ = io.Copy(ioutil.Discard, res.Body)
+		_ = res.Body.Close()
+	}()
 
-	switch res.StatusCode {
+	return n.determineUploadState(res.StatusCode, log, messagePrefix)
+}
+
+func (n *GitLabClient) determineUploadState(
+	statusCode int,
+	log *logrus.Entry,
+	messagePrefix string,
+) common.UploadState {
+	switch statusCode {
 	case http.StatusCreated:
 		log.Println(messagePrefix, "ok")
 		return common.UploadSucceeded
 	case http.StatusForbidden:
-		log.WithField("status", res.Status).Errorln(messagePrefix, "forbidden")
+		log.WithField("status", statusCode).Errorln(messagePrefix, "forbidden")
 		return common.UploadForbidden
 	case http.StatusRequestEntityTooLarge:
-		log.WithField("status", res.Status).Errorln(messagePrefix, "too large archive")
+		log.WithField("status", statusCode).Errorln(messagePrefix, "too large archive")
 		return common.UploadTooLarge
 	case http.StatusServiceUnavailable:
-		log.WithField("status", res.Status).Errorln(messagePrefix, "service unavailable")
+		log.WithField("status", statusCode).Errorln(messagePrefix, "service unavailable")
 		return common.UploadServiceUnavailable
 	default:
-		log.WithField("status", res.Status).Warningln(messagePrefix, "failed")
+		log.WithField("status", statusCode).Warningln(messagePrefix, "failed")
 		return common.UploadFailed
 	}
 }
 
-func (n *GitLabClient) DownloadArtifacts(config common.JobCredentials, artifactsFile string, directDownload *bool) common.DownloadState {
+func (n *GitLabClient) DownloadArtifacts(
+	config common.JobCredentials,
+	artifactsFile string,
+	directDownload *bool,
+) common.DownloadState {
 	query := url.Values{}
 
 	if directDownload != nil {
@@ -572,19 +698,21 @@ func (n *GitLabClient) DownloadArtifacts(config common.JobCredentials, artifacts
 		log.Errorln("Downloading artifacts from coordinator...", "error", err.Error())
 		return common.DownloadFailed
 	}
-	defer res.Body.Close()
-	defer io.Copy(ioutil.Discard, res.Body)
+	defer func() {
+		_, _ = io.Copy(ioutil.Discard, res.Body)
+		_ = res.Body.Close()
+	}()
 
 	switch res.StatusCode {
 	case http.StatusOK:
 		file, err := os.Create(artifactsFile)
 		if err == nil {
-			defer file.Close()
+			defer func() { _ = file.Close() }()
 			_, err = io.Copy(file, res.Body)
 		}
 		if err != nil {
-			file.Close()
-			os.Remove(file.Name())
+			_ = file.Close()
+			_ = os.Remove(file.Name())
 			log.WithError(err).Errorln("Downloading artifacts from coordinator...", "error")
 			return common.DownloadFailed
 		}
@@ -602,7 +730,10 @@ func (n *GitLabClient) DownloadArtifacts(config common.JobCredentials, artifacts
 	}
 }
 
-func (n *GitLabClient) ProcessJob(config common.RunnerConfig, jobCredentials *common.JobCredentials) (common.JobTrace, error) {
+func (n *GitLabClient) ProcessJob(
+	config common.RunnerConfig,
+	jobCredentials *common.JobCredentials,
+) (common.JobTrace, error) {
 	trace, err := newJobTrace(n, config, jobCredentials)
 	if err != nil {
 		return nil, err
