@@ -9,10 +9,6 @@ import (
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/trace"
 )
 
-const (
-	emptyRemoteTraceUpdateInterval = 0
-)
-
 type clientJobTrace struct {
 	client         common.Network
 	config         common.RunnerConfig
@@ -30,19 +26,18 @@ type clientJobTrace struct {
 	sentTrace int
 	sentTime  time.Time
 
-	updateInterval      time.Duration
-	forceSendInterval   time.Duration
-	finishRetryInterval time.Duration
-	maxTracePatchSize   int
+	updateInterval    time.Duration
+	forceSendInterval time.Duration
+	maxTracePatchSize int
 
 	failuresCollector common.FailuresCollector
 }
 
 func (c *clientJobTrace) Success() {
-	c.Fail(nil, common.NoneFailure)
+	c.complete(nil, "")
 }
 
-func (c *clientJobTrace) Fail(err error, failureReason common.JobFailureReason) {
+func (c *clientJobTrace) complete(err error, failureReason common.JobFailureReason) {
 	c.lock.Lock()
 
 	if c.state != common.Running {
@@ -60,6 +55,10 @@ func (c *clientJobTrace) Fail(err error, failureReason common.JobFailureReason)
 	c.finish()
 }
 
+func (c *clientJobTrace) Fail(err error, failureReason common.JobFailureReason) {
+	c.complete(err, failureReason)
+}
+
 func (c *clientJobTrace) Write(data []byte) (n int, err error) {
 	return c.buffer.Write(data)
 }
@@ -68,10 +67,29 @@ func (c *clientJobTrace) SetMasked(masked []string) {
 	c.buffer.SetMasked(masked)
 }
 
+func (c *clientJobTrace) checksum() string {
+	return c.buffer.Checksum()
+}
+
 func (c *clientJobTrace) SetCancelFunc(cancelFunc context.CancelFunc) {
+	c.lock.Lock()
+	defer c.lock.Unlock()
+
 	c.cancelFunc = cancelFunc
 }
 
+func (c *clientJobTrace) Cancel() bool {
+	c.lock.RLock()
+	defer c.lock.RUnlock()
+
+	if c.cancelFunc == nil {
+		return false
+	}
+
+	c.cancelFunc()
+	return true
+}
+
 func (c *clientJobTrace) SetFailuresCollector(fc common.FailuresCollector) {
 	c.failuresCollector = fc
 }
@@ -95,26 +113,34 @@ func (c *clientJobTrace) start() {
 	go c.watch()
 }
 
-func (c *clientJobTrace) finalTraceUpdate() {
+func (c *clientJobTrace) ensureAllTraceSent() {
 	for c.anyTraceToSend() {
 		switch c.sendPatch() {
-		case common.UpdateSucceeded:
+		case common.PatchSucceeded:
 			// we continue sending till we succeed
 			continue
-		case common.UpdateAbort:
+		case common.PatchAbort:
 			return
-		case common.UpdateNotFound:
+		case common.PatchNotFound:
 			return
-		case common.UpdateRangeMismatch:
-			time.Sleep(c.finishRetryInterval)
-		case common.UpdateFailed:
-			time.Sleep(c.finishRetryInterval)
+		case common.PatchRangeMismatch:
+			time.Sleep(c.getUpdateInterval())
+		case common.PatchFailed:
+			time.Sleep(c.getUpdateInterval())
 		}
 	}
 }
 
-func (c *clientJobTrace) finalStatusUpdate() {
+func (c *clientJobTrace) finalUpdate() {
+	// On final-update we want the Runner to fallback
+	// to default interval and make Rails to override it
+	c.setUpdateInterval(common.DefaultUpdateInterval)
+
 	for {
+		// Before sending update to ensure that trace is sent
+		// as `sendUpdate()` can force Runner to rewind trace
+		c.ensureAllTraceSent()
+
 		switch c.sendUpdate() {
 		case common.UpdateSucceeded:
 			return
@@ -122,10 +148,12 @@ func (c *clientJobTrace) finalStatusUpdate() {
 			return
 		case common.UpdateNotFound:
 			return
-		case common.UpdateRangeMismatch:
-			return
+		case common.UpdateAcceptedButNotCompleted:
+			time.Sleep(c.getUpdateInterval())
+		case common.UpdateTraceValidationFailed:
+			time.Sleep(c.getUpdateInterval())
 		case common.UpdateFailed:
-			time.Sleep(c.finishRetryInterval)
+			time.Sleep(c.getUpdateInterval())
 		}
 	}
 }
@@ -133,18 +161,31 @@ func (c *clientJobTrace) finalStatusUpdate() {
 func (c *clientJobTrace) finish() {
 	c.buffer.Finish()
 	c.finished <- true
-	c.finalTraceUpdate()
-	c.finalStatusUpdate()
+	c.finalUpdate()
 	c.buffer.Close()
 }
 
-func (c *clientJobTrace) incrementalUpdate() common.UpdateState {
-	state := c.sendPatch()
-	if state != common.UpdateSucceeded {
-		return state
+// incrementalUpdate returns a flag if jobs is supposed
+// to be running, or whether it should be finished
+func (c *clientJobTrace) incrementalUpdate() bool {
+	patchState := c.sendPatch()
+
+	if patchState == common.PatchSucceeded {
+		// We try to additionally touch job to check
+		// it might be required if no content was send
+		// for longer period of time.
+		// This is needed to discover if it should be aborted
+		touchState := c.touchJob()
+
+		// Try to abort job
+		if touchState == common.UpdateAbort && c.abort() {
+			return false
+		}
+	} else if patchState == common.PatchAbort && c.abort() {
+		return false
 	}
 
-	return c.touchJob()
+	return true
 }
 
 func (c *clientJobTrace) anyTraceToSend() bool {
@@ -154,25 +195,25 @@ func (c *clientJobTrace) anyTraceToSend() bool {
 	return c.buffer.Size() != c.sentTrace
 }
 
-func (c *clientJobTrace) sendPatch() common.UpdateState {
+func (c *clientJobTrace) sendPatch() common.PatchState {
 	c.lock.RLock()
 	content, err := c.buffer.Bytes(c.sentTrace, c.maxTracePatchSize)
 	sentTrace := c.sentTrace
 	c.lock.RUnlock()
 
 	if err != nil {
-		return common.UpdateFailed
+		return common.PatchFailed
 	}
 
 	if len(content) == 0 {
-		return common.UpdateSucceeded
+		return common.PatchSucceeded
 	}
 
 	result := c.client.PatchTrace(c.config, c.jobCredentials, content, sentTrace)
 
 	c.setUpdateInterval(result.NewUpdateInterval)
 
-	if result.State == common.UpdateSucceeded || result.State == common.UpdateRangeMismatch {
+	if result.State == common.PatchSucceeded || result.State == common.PatchRangeMismatch {
 		c.lock.Lock()
 		c.sentTime = time.Now()
 		c.sentTrace = result.SentOffset
@@ -183,7 +224,7 @@ func (c *clientJobTrace) sendPatch() common.UpdateState {
 }
 
 func (c *clientJobTrace) setUpdateInterval(newUpdateInterval time.Duration) {
-	if newUpdateInterval <= time.Duration(emptyRemoteTraceUpdateInterval) {
+	if newUpdateInterval <= 0 {
 		return
 	}
 
@@ -191,6 +232,12 @@ func (c *clientJobTrace) setUpdateInterval(newUpdateInterval time.Duration) {
 	defer c.lock.Unlock()
 
 	c.updateInterval = newUpdateInterval
+
+	// Let's hope that this never happens,
+	// but if server behaves bogus do not have too long interval
+	if c.updateInterval > common.MaxUpdateInterval {
+		c.updateInterval = common.MaxUpdateInterval
+	}
 }
 
 // Update Coordinator that the job is still running.
@@ -204,19 +251,22 @@ func (c *clientJobTrace) touchJob() common.UpdateState {
 	}
 
 	jobInfo := common.UpdateJobInfo{
-		ID:    c.id,
-		State: common.Running,
+		ID:       c.id,
+		State:    common.Running,
+		Checksum: c.checksum(),
 	}
 
-	status := c.client.UpdateJob(c.config, c.jobCredentials, jobInfo)
+	result := c.client.UpdateJob(c.config, c.jobCredentials, jobInfo)
 
-	if status == common.UpdateSucceeded {
+	c.setUpdateInterval(result.NewUpdateInterval)
+
+	if result.State == common.UpdateSucceeded {
 		c.lock.Lock()
 		c.sentTime = time.Now()
 		c.lock.Unlock()
 	}
 
-	return status
+	return result.State
 }
 
 func (c *clientJobTrace) sendUpdate() common.UpdateState {
@@ -228,38 +278,42 @@ func (c *clientJobTrace) sendUpdate() common.UpdateState {
 		ID:            c.id,
 		State:         state,
 		FailureReason: c.failureReason,
+		Checksum:      c.checksum(),
 	}
 
-	status := c.client.UpdateJob(c.config, c.jobCredentials, jobInfo)
+	result := c.client.UpdateJob(c.config, c.jobCredentials, jobInfo)
 
-	if status == common.UpdateSucceeded {
+	c.setUpdateInterval(result.NewUpdateInterval)
+
+	if result.State == common.UpdateSucceeded {
 		c.lock.Lock()
 		c.sentTime = time.Now()
 		c.lock.Unlock()
+	} else if result.State == common.UpdateTraceValidationFailed {
+		c.lock.Lock()
+		c.sentTime = time.Now()
+		c.sentTrace = 0
+		c.lock.Unlock()
 	}
 
-	return status
+	return result.State
 }
 
 func (c *clientJobTrace) abort() bool {
-	if c.cancelFunc != nil {
-		c.cancelFunc()
-		c.cancelFunc = nil
-		return true
-	}
-	return false
+	cancelled := c.Cancel()
+	c.SetCancelFunc(nil)
+	return cancelled
 }
 
 func (c *clientJobTrace) watch() {
 	for {
 		select {
 		case <-time.After(c.getUpdateInterval()):
-			state := c.incrementalUpdate()
-			if state == common.UpdateAbort && c.abort() {
+			if !c.incrementalUpdate() {
+				// job is no longer running, wait for finish
 				<-c.finished
 				return
 			}
-			break
 
 		case <-c.finished:
 			return
@@ -283,21 +337,24 @@ func (c *clientJobTrace) setupLogLimit() {
 	c.buffer.SetLimit(bytesLimit)
 }
 
-func newJobTrace(client common.Network, config common.RunnerConfig, jobCredentials *common.JobCredentials) (*clientJobTrace, error) {
+func newJobTrace(
+	client common.Network,
+	config common.RunnerConfig,
+	jobCredentials *common.JobCredentials,
+) (*clientJobTrace, error) {
 	buffer, err := trace.New()
 	if err != nil {
 		return nil, err
 	}
 
 	return &clientJobTrace{
-		client:              client,
-		config:              config,
-		buffer:              buffer,
-		jobCredentials:      jobCredentials,
-		id:                  jobCredentials.ID,
-		maxTracePatchSize:   common.DefaultTracePatchLimit,
-		updateInterval:      common.DefaultTraceUpdateInterval,
-		forceSendInterval:   common.TraceForceSendInterval,
-		finishRetryInterval: common.TraceFinishRetryInterval,
+		client:            client,
+		config:            config,
+		buffer:            buffer,
+		jobCredentials:    jobCredentials,
+		id:                jobCredentials.ID,
+		maxTracePatchSize: common.DefaultTracePatchLimit,
+		updateInterval:    common.DefaultUpdateInterval,
+		forceSendInterval: common.TraceForceSendInterval,
 	}, nil
 }
